@@ -13,7 +13,7 @@ use std::io::{BufRead, BufReader};
 use std::path::{Component, Path, PathBuf};
 use std::process::{Command, Stdio};
 use toml::Value;
-use tracing::{error, info, trace, warn};
+use tracing::{debug, error, info, trace, warn};
 use walkdir::{DirEntry, WalkDir};
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, Ord, PartialOrd)]
@@ -48,6 +48,7 @@ pub struct TestBinary {
     pkg_version: Option<String>,
     pkg_authors: Option<Vec<String>>,
     should_panic: bool,
+    linker_paths: Vec<PathBuf>,
 }
 
 #[derive(Clone, Debug)]
@@ -66,6 +67,7 @@ impl TestBinary {
             pkg_authors: None,
             cargo_dir: None,
             should_panic: false,
+            linker_paths: vec![],
         }
     }
 
@@ -93,6 +95,30 @@ impl TestBinary {
         &self.pkg_authors
     }
 
+    pub fn has_linker_paths(&self) -> bool {
+        !self.linker_paths.is_empty()
+    }
+
+    pub fn is_test_type(&self) -> bool {
+        matches!(self.ty, None | Some(RunType::Tests))
+    }
+
+    /// Convert linker paths to an LD_LIBRARY_PATH.
+    /// TODO this won't work for windows when it's implemented
+    pub fn ld_library_path(&self) -> String {
+        let mut new_vals = self
+            .linker_paths
+            .iter()
+            .map(|x| x.display().to_string())
+            .collect::<Vec<String>>()
+            .join(":");
+        if let Ok(ld) = env::var("LD_LIBRARY_PATH") {
+            new_vals.push(':');
+            new_vals.push_str(ld.as_str());
+        }
+        new_vals
+    }
+
     /// Should be `false` for normal tests and for doctests either `true` or
     /// `false` depending on the test attribute
     pub fn should_panic(&self) -> bool {
@@ -185,10 +211,7 @@ pub fn get_tests(config: &Config) -> Result<Vec<TestBinary>, RunError> {
             }
         }
     }
-    let manifest = match config.manifest.as_path().to_str() {
-        Some(s) => s,
-        None => "Cargo.toml",
-    };
+    let manifest = config.manifest.as_path().to_str().unwrap_or("Cargo.toml");
     let metadata = MetadataCommand::new()
         .manifest_path(manifest)
         .features(CargoOpt::AllFeatures)
@@ -227,6 +250,8 @@ fn run_cargo(
     }
     trace!("Running command {:?}", cmd);
     let mut child = cmd.spawn().map_err(|e| RunError::Cargo(e.to_string()))?;
+    let update_from = result.len();
+    let mut paths = vec![];
 
     if ty != Some(RunType::Doctests) {
         let mut package_ids = vec![];
@@ -251,12 +276,39 @@ fn run_cargo(
                     }
                     _ => {}
                 },
+                Ok(Message::BuildScriptExecuted(bs))
+                    if !(bs.linked_libs.is_empty() || bs.linked_paths.is_empty()) =>
+                {
+                    let mut temp_paths = bs
+                        .linked_paths
+                        .iter()
+                        .filter_map(|x| {
+                            if x.as_std_path().exists() {
+                                Some(x.as_std_path().to_path_buf())
+                            } else if let Some(index) = x.as_str().find("=") {
+                                Some(PathBuf::from(&x.as_str()[(index + 1)..]))
+                            } else {
+                                warn!("Couldn't resolve linker path: {}", x.as_str());
+                                None
+                            }
+                        })
+                        .collect::<Vec<PathBuf>>();
+                    for p in temp_paths.drain(..) {
+                        if !paths.contains(&p) {
+                            paths.push(p);
+                        }
+                    }
+                }
                 Err(e) => {
                     error!("Error parsing cargo messages {}", e);
                 }
                 _ => {}
             }
         }
+        debug!("Linker paths: {:?}", paths);
+        for bin in result.iter_mut().skip(update_from) {
+            bin.linker_paths = paths.clone();
+        }
         let status = child.wait().unwrap();
         if let Some(error) = error {
             return Err(error);
@@ -290,9 +342,24 @@ fn run_cargo(
             .filter_map(|e| e.ok())
             .filter(|e| matches!(e.metadata(), Ok(ref m) if m.is_file() && m.len() != 0))
             .collect::<Vec<_>>();
-        let should_panics = get_panic_candidates(&dir_entries, config);
+        let should_panics = get_attribute_candidates(&dir_entries, config, "should_panic");
+        let no_runs = get_attribute_candidates(&dir_entries, config, "no_run");
         for dt in &dir_entries {
             let mut tb = TestBinary::new(dt.path().to_path_buf(), ty);
+
+            if let Some(meta) = DocTestBinaryMeta::new(dt.path()) {
+                if no_runs
+                    .get(&meta.prefix)
+                    .map(|x| x.contains(&meta.line))
+                    .unwrap_or(false)
+                {
+                    info!("Skipping no_run doctest: {}", dt.path().display());
+                    continue;
+                }
+                if let Some(lines) = should_panics.get(&meta.prefix) {
+                    tb.should_panic |= lines.contains(&meta.line);
+                }
+            }
             let mut current_dir = dt.path();
             loop {
                 if current_dir.is_dir() && current_dir.join("Cargo.toml").exists() {
@@ -306,12 +373,6 @@ fn run_cargo(
                     None => break,
                 }
             }
-            // Now to do my magic!
-            if let Some(meta) = DocTestBinaryMeta::new(dt.path()) {
-                if let Some(lines) = should_panics.get(&meta.prefix) {
-                    tb.should_panic |= lines.contains(&meta.line);
-                }
-            }
             result.push(tb);
         }
     }
@@ -356,10 +417,14 @@ fn is_prefix_match(prefix: &str, entry: &Path) -> bool {
 ///
 /// Currently all doctest files take the pattern of `{name}_{line}_{number}` where name is the
 /// path to the file with directory separators and dots replaced with underscores. Therefore
-/// each name could potentially map to many files as `src_some_folder_foo_rs_0_1` could go to
+/// each name could potentially map to many files as `src_some_folder_foo_rs_1_1` could go to
 /// `src/some/folder_foo.rs` or `src/some/folder/foo.rs` here we're going to work on a heuristic
 /// that any matching file is good because we can't do any better
-fn get_panic_candidates(tests: &[DirEntry], config: &Config) -> HashMap<String, Vec<usize>> {
+fn get_attribute_candidates(
+    tests: &[DirEntry],
+    config: &Config,
+    attribute: &str,
+) -> HashMap<String, Vec<usize>> {
     let mut result = HashMap::new();
     let mut checked_files = HashSet::new();
     let root = config.root();
@@ -372,7 +437,7 @@ fn get_panic_candidates(tests: &[DirEntry], config: &Config) -> HashMap<String,
                         if is_prefix_match(&test_binary.prefix, &p) && !checked_files.contains(path)
                         {
                             checked_files.insert(path.to_path_buf());
-                            let lines = find_panics_in_file(path).unwrap_or_default();
+                            let lines = find_str_in_file(path, attribute).unwrap_or_default();
                             if !result.contains_key(&test_binary.prefix) {
                                 result.insert(test_binary.prefix.clone(), lines);
                             } else if let Some(current_lines) = result.get_mut(&test_binary.prefix)
@@ -393,17 +458,13 @@ fn get_panic_candidates(tests: &[DirEntry], config: &Config) -> HashMap<String,
     result
 }
 
-fn find_panics_in_file(file: &Path) -> io::Result<Vec<usize>> {
+fn find_str_in_file(file: &Path, value: &str) -> io::Result<Vec<usize>> {
     let f = File::open(file)?;
     let reader = BufReader::new(f);
     let lines = reader
         .lines()
         .enumerate()
-        .filter(|(_, l)| {
-            l.as_ref()
-                .map(|x| x.contains("should_panic"))
-                .unwrap_or(false)
-        })
+        .filter(|(_, l)| l.as_ref().map(|x| x.contains(value)).unwrap_or(false))
         .map(|(i, _)| i + 1) // Move from line index to line number
         .collect();
     Ok(lines)
@@ -559,17 +620,18 @@ fn clean_doctest_folder<P: AsRef<Path>>(doctest_dir: P) {
 
 fn handle_llvm_flags(value: &mut String, config: &Config) {
     if config.engine() == TraceEngine::Llvm {
-        value.push_str("-Z instrument-coverage ");
-    } else {
-        value.push_str(" -C link-dead-code ");
+        value.push_str("-Zinstrument-coverage ");
+    }
+    if cfg!(not(windows)) {
+        value.push_str(" -Clink-dead-code ");
     }
 }
 
 pub fn rustdoc_flags(config: &Config) -> String {
     const RUSTDOC: &str = "RUSTDOCFLAGS";
-    let common_opts = " -C link-dead-code -C debuginfo=2 --cfg=tarpaulin ";
+    let common_opts = " -Clink-dead-code -Cdebuginfo=2 --cfg=tarpaulin ";
     let mut value = format!(
-        "{} --persist-doctests {} -Z unstable-options ",
+        "{} --persist-doctests {} -Zunstable-options ",
         common_opts,
         config.doctest_dir().display()
     );
@@ -579,7 +641,7 @@ pub fn rustdoc_flags(config: &Config) -> String {
         }
     }
     handle_llvm_flags(&mut value, config);
-    value
+    deduplicate_flags(&value)
 }
 
 fn look_for_rustflags_in_table(value: &Value) -> String {
@@ -590,7 +652,7 @@ fn look_for_rustflags_in_table(value: &Value) -> String {
             let vec_of_flags: Vec<String> = rustflags
                 .as_array()
                 .unwrap()
-                .into_iter()
+                .iter()
                 .filter_map(|x| x.as_str())
                 .map(|x| x.to_string())
                 .collect();
@@ -672,12 +734,12 @@ fn gather_config_rust_flags(config: &Config) -> String {
 pub fn rust_flags(config: &Config) -> String {
     const RUSTFLAGS: &str = "RUSTFLAGS";
     let mut value = config.rustflags.clone().unwrap_or_default();
-    value.push_str(" -C debuginfo=2 ");
+    value.push_str(" -Cdebuginfo=2 ");
     if !config.avoid_cfg_tarpaulin {
         value.push_str("--cfg=tarpaulin ");
     }
     if config.release {
-        value.push_str("-C debug-assertions=off ");
+        value.push_str("-Cdebug-assertions=off ");
     }
     handle_llvm_flags(&mut value, config);
     lazy_static! {
@@ -689,7 +751,38 @@ pub fn rust_flags(config: &Config) -> String {
         let vtemp = gather_config_rust_flags(config);
         value.push_str(&DEBUG_INFO.replace_all(&vtemp, " "));
     }
-    value
+    deduplicate_flags(&value)
+}
+
+fn deduplicate_flags(flags: &str) -> String {
+    lazy_static! {
+        static ref CFG_FLAG: Regex = Regex::new(r#"\--cfg\s+"#).unwrap();
+        static ref C_FLAG: Regex = Regex::new(r#"\-C\s+"#).unwrap();
+        static ref Z_FLAG: Regex = Regex::new(r#"\-Z\s+"#).unwrap();
+    }
+
+    // Gonna remove the excess spaces to make it easier to filter things
+    let res = CFG_FLAG.replace_all(&flags, "--cfg=");
+    let res = C_FLAG.replace_all(&res, "-C");
+    let res = Z_FLAG.replace_all(&res, "-Z");
+
+    let mut flag_set = HashSet::new();
+    let mut result = vec![];
+    for val in res.split_whitespace() {
+        if val.starts_with("--cfg") {
+            if !flag_set.contains(&val) {
+                result.push(val);
+                flag_set.insert(val);
+            }
+        } else {
+            let id = val.split("=").next().unwrap();
+            if !flag_set.contains(id) {
+                flag_set.insert(id);
+                result.push(val);
+            }
+        }
+    }
+    result.join(" ")
 }
 
 fn setup_environment(cmd: &mut Command, config: &Config) {
@@ -770,4 +863,29 @@ mod tests {
         version.channel = Channel::Stable;
         assert!(!version.supports_llvm_cov());
     }
+
+    #[test]
+    fn no_duplicate_flags() {
+        assert_eq!(
+            deduplicate_flags("--cfg=tarpaulin --cfg tarpaulin"),
+            "--cfg=tarpaulin"
+        );
+        assert_eq!(
+            deduplicate_flags("-Clink-dead-code -Zinstrument-coverage -C link-dead-code"),
+            "-Clink-dead-code -Zinstrument-coverage"
+        );
+        assert_eq!(
+            deduplicate_flags("-Clink-dead-code -Zinstrument-coverage -Zinstrument-coverage"),
+            "-Clink-dead-code -Zinstrument-coverage"
+        );
+        assert_eq!(
+            deduplicate_flags("-Clink-dead-code -Zinstrument-coverage -Cinstrument-coverage"),
+            "-Clink-dead-code -Zinstrument-coverage -Cinstrument-coverage"
+        );
+
+        assert_eq!(
+            deduplicate_flags("--cfg=tarpaulin --cfg tarpauline --cfg=tarp"),
+            "--cfg=tarpaulin --cfg=tarpauline --cfg=tarp"
+        );
+    }
 }
