@@ -166,6 +166,9 @@ pub struct Config {
     pub follow_exec: bool,
     /// Number of jobs used for building the tests
     pub jobs: Option<usize>,
+    /// Allow test to use an implicit test threads
+    #[serde(rename = "implicit-test-threads")]
+    pub implicit_test_threads: bool,
     /// Engine to use to collect coverage
     engine: RefCell<TraceEngine>,
     /// Specifying per-config rust flags
@@ -218,6 +221,7 @@ impl Default for Config {
             no_run: false,
             locked: false,
             frozen: false,
+            implicit_test_threads: false,
             target: None,
             target_dir: None,
             offline: false,
@@ -321,6 +325,7 @@ impl<'a> From<&'a ArgMatches<'a>> for ConfigWrapper {
             profile: get_profile(args),
             metadata: RefCell::new(None),
             avoid_cfg_tarpaulin: args.is_present("avoid-cfg-tarpaulin"),
+            implicit_test_threads: args.is_present("implicit-test-threads"),
             rustflags: get_rustflags(args),
         };
         if args.is_present("ignore-config") {
@@ -490,7 +495,7 @@ impl Config {
     }
 
     pub fn parse_config_toml(buffer: &[u8]) -> std::io::Result<Vec<Self>> {
-        let mut map: IndexMap<String, Self> = toml::from_slice(&buffer).map_err(|e| {
+        let mut map: IndexMap<String, Self> = toml::from_slice(buffer).map_err(|e| {
             error!("Invalid config file {}", e);
             Error::new(ErrorKind::InvalidData, format!("{}", e))
         })?;
@@ -524,6 +529,7 @@ impl Config {
         self.release |= other.release;
         self.count |= other.count;
         self.all_features |= other.all_features;
+        self.implicit_test_threads |= other.implicit_test_threads;
         self.all_targets |= other.all_targets;
         self.line_coverage |= other.line_coverage;
         self.branch_coverage |= other.branch_coverage;
@@ -913,7 +919,7 @@ mod tests {
         assert_eq!(configs.len(), 2);
         for c in &configs {
             if c.name == "global" {
-                assert_eq!(c.run_ignored, true);
+                assert!(c.run_ignored);
                 assert_eq!(c.coveralls, Some("hello".to_string()));
             } else if c.name == "other" {
                 assert_eq!(c.run_types, vec![RunType::Doctests, RunType::Tests]);
@@ -970,11 +976,11 @@ mod tests {
         let mut a: Config = toml::from_slice(toml_a.as_bytes()).unwrap();
         let b: Config = toml::from_slice(toml_b.as_bytes()).unwrap();
 
-        assert_eq!(a.all, false);
-        assert_eq!(b.all, true);
+        assert!(!a.all);
+        assert!(b.all);
 
         a.merge(&b);
-        assert_eq!(a.all, true);
+        assert!(a.all);
     }
 
     #[test]
@@ -1115,7 +1121,7 @@ mod tests {
         no_yes.merge(&flag2);
         assert_eq!(no_yes.rustflags, Some("bar".to_string()));
 
-        let mut f1_2 = flag1.clone();
+        let mut f1_2 = flag1;
         f1_2.merge(&flag2);
         let flags = f1_2.rustflags.unwrap();
         let split = flags.split_ascii_whitespace().collect::<Vec<_>>();
