@@ -809,6 +809,11 @@ fn tarpaulin_skip_attr() {
         fn uncovered2() {
             println!(\"oof\");
         }
+
+        #[no_coverage]
+        fn uncovered3() {
+            println!(\"zombie lincoln\");
+        }
         ",
         file: Path::new(""),
         ignore_mods: RefCell::new(HashSet::new()),
@@ -825,6 +830,8 @@ fn tarpaulin_skip_attr() {
     assert!(lines.ignore.contains(&Lines::Line(13)));
     assert!(lines.ignore.contains(&Lines::Line(17)));
     assert!(lines.ignore.contains(&Lines::Line(18)));
+    assert!(lines.ignore.contains(&Lines::Line(22)));
+    assert!(lines.ignore.contains(&Lines::Line(23)));
 
     let ctx = Context {
         config: &config,
@@ -1227,3 +1234,163 @@ fn unreachable_propagate() {
     assert!(lines.ignore.contains(&Lines::Line(6)));
     assert!(lines.ignore.contains(&Lines::Line(7)));
 }
+
+#[test]
+fn unreachable_include_returns() {
+    let config = Config::default();
+    let ctx = Context {
+        config: &config,
+        file_contents: "fn test_not_unreachable() -> Result<(), Box<dyn std::error::Error>> {
+            let x: u32 = foo();
+            if x > 5 {
+                bar();
+                return true;
+            }
+            std::fs::remove_dir(\"I don't exist and will definitely fail/so yeahhhh...\")?;
+            unreachable!();
+        }",
+        file: Path::new(""),
+        ignore_mods: RefCell::new(HashSet::new()),
+    };
+    let parser = parse_file(ctx.file_contents).unwrap();
+    let mut analysis = SourceAnalysis::new();
+    analysis.process_items(&parser.items, &ctx);
+    let lines = analysis.get_line_analysis(ctx.file.to_path_buf());
+    assert!(!lines.ignore.contains(&Lines::Line(1)));
+    assert!(!lines.ignore.contains(&Lines::Line(2)));
+    assert!(!lines.ignore.contains(&Lines::Line(3)));
+    assert!(!lines.ignore.contains(&Lines::Line(4)));
+    assert!(!lines.ignore.contains(&Lines::Line(5)));
+    assert!(!lines.ignore.contains(&Lines::Line(6)));
+    assert!(!lines.ignore.contains(&Lines::Line(7)));
+    assert!(lines.ignore.contains(&Lines::Line(8)));
+
+    let ctx = Context {
+        config: &config,
+        file_contents: "fn excluded_from_coverage(option: bool) -> bool {
+            if option {
+                return true;
+            }
+            if !option {
+                return false;
+            }
+            unreachable!();
+        }
+        ",
+        file: Path::new(""),
+        ignore_mods: RefCell::new(HashSet::new()),
+    };
+    let parser = parse_file(ctx.file_contents).unwrap();
+    let mut analysis = SourceAnalysis::new();
+    analysis.process_items(&parser.items, &ctx);
+    let lines = analysis.get_line_analysis(ctx.file.to_path_buf());
+    assert!(!lines.ignore.contains(&Lines::Line(1)));
+    assert!(!lines.ignore.contains(&Lines::Line(2)));
+    assert!(!lines.ignore.contains(&Lines::Line(3)));
+    assert!(!lines.ignore.contains(&Lines::Line(4)));
+    assert!(!lines.ignore.contains(&Lines::Line(5)));
+    assert!(!lines.ignore.contains(&Lines::Line(6)));
+    assert!(!lines.ignore.contains(&Lines::Line(7)));
+    assert!(lines.ignore.contains(&Lines::Line(8)));
+}
+
+#[test]
+fn unreachable_include_loops() {
+    let config = Config::default();
+    let ctx = Context {
+        config: &config,
+        file_contents: "fn test_not_unreachable() {
+            loop {
+                bar();
+            }
+            unreachable!();
+        }",
+        file: Path::new(""),
+        ignore_mods: RefCell::new(HashSet::new()),
+    };
+    let parser = parse_file(ctx.file_contents).unwrap();
+    let mut analysis = SourceAnalysis::new();
+    analysis.process_items(&parser.items, &ctx);
+    let lines = analysis.get_line_analysis(ctx.file.to_path_buf());
+    assert!(!lines.ignore.contains(&Lines::Line(1)));
+    assert!(!lines.ignore.contains(&Lines::Line(2)));
+    assert!(!lines.ignore.contains(&Lines::Line(3)));
+    assert!(!lines.ignore.contains(&Lines::Line(4)));
+    assert!(lines.ignore.contains(&Lines::Line(5)));
+
+    let ctx = Context {
+        config: &config,
+        file_contents: "fn test_not_unreachable() {
+            while true {
+                bar();
+            }
+            unreachable!();
+        }",
+        file: Path::new(""),
+        ignore_mods: RefCell::new(HashSet::new()),
+    };
+    let parser = parse_file(ctx.file_contents).unwrap();
+    let mut analysis = SourceAnalysis::new();
+    analysis.process_items(&parser.items, &ctx);
+    let lines = analysis.get_line_analysis(ctx.file.to_path_buf());
+    assert!(!lines.ignore.contains(&Lines::Line(1)));
+    assert!(!lines.ignore.contains(&Lines::Line(2)));
+    assert!(!lines.ignore.contains(&Lines::Line(3)));
+    assert!(!lines.ignore.contains(&Lines::Line(4)));
+    assert!(lines.ignore.contains(&Lines::Line(5)));
+
+    let ctx = Context {
+        config: &config,
+        file_contents: "fn test_not_unreachable() -> usize {
+            for i in &[1,2,3,4] {
+                return *i;
+            }
+            unreachable!();
+        }",
+        file: Path::new(""),
+        ignore_mods: RefCell::new(HashSet::new()),
+    };
+    let parser = parse_file(ctx.file_contents).unwrap();
+    let mut analysis = SourceAnalysis::new();
+    analysis.process_items(&parser.items, &ctx);
+    let lines = analysis.get_line_analysis(ctx.file.to_path_buf());
+    assert!(!lines.ignore.contains(&Lines::Line(1)));
+    assert!(!lines.ignore.contains(&Lines::Line(2)));
+    assert!(!lines.ignore.contains(&Lines::Line(3)));
+    assert!(!lines.ignore.contains(&Lines::Line(4)));
+    assert!(lines.ignore.contains(&Lines::Line(5)));
+}
+
+#[test]
+fn single_line_callables() {
+    let config = Config::default();
+    let ctx = Context {
+        config: &config,
+        file_contents: "struct A;
+        impl A {
+        fn foo() {}
+        fn bar(i: i32) {}
+        }
+
+        fn foo() {}
+        fn bar(i: i32) {}
+
+        fn blah() {
+             foo();
+             A::foo();
+             bar(2);
+             A::bar(2);
+        }
+        ",
+        file: Path::new(""),
+        ignore_mods: RefCell::new(HashSet::new()),
+    };
+    let parser = parse_file(ctx.file_contents).unwrap();
+    let mut analysis = SourceAnalysis::new();
+    analysis.process_items(&parser.items, &ctx);
+    let lines = analysis.get_line_analysis(ctx.file.to_path_buf());
+    assert!(!lines.ignore.contains(&Lines::Line(11)));
+    assert!(!lines.ignore.contains(&Lines::Line(12)));
+    assert!(!lines.ignore.contains(&Lines::Line(13)));
+    assert!(!lines.ignore.contains(&Lines::Line(14)));
+}
